@@ -17,10 +17,7 @@ import unittest
 
 import mock
 
-try:
-    from google.cloud import bigquery_storage
-except ImportError:  # pragma: NO COVER
-    bigquery_storage = None
+from google.cloud import bigquery_storage
 
 
 class TestConnection(unittest.TestCase):
@@ -40,8 +37,6 @@ class TestConnection(unittest.TestCase):
         return mock_client
 
     def _mock_bqstorage_client(self):
-        # Assumption: bigquery_storage exists. It's the test's responisbility to
-        # not use this helper or skip itself if bqstroage is not installed.
         mock_client = mock.create_autospec(bigquery_storage.BigQueryReadClient)
         mock_client._transport = mock.Mock(spec=["channel"])
         mock_client._transport.grpc_channel = mock.Mock(spec=["close"])
@@ -58,9 +53,6 @@ class TestConnection(unittest.TestCase):
         self.assertIs(connection._client, mock_client)
         self.assertIs(connection._bqstorage_client, None)
 
-    @unittest.skipIf(
-        bigquery_storage is None, "Requires `google-cloud-bigquery-storage`"
-    )
     def test_ctor_w_bqstorage_client(self):
         from google.cloud.bigquery.dbapi import Connection
 
@@ -90,9 +82,6 @@ class TestConnection(unittest.TestCase):
         self.assertIsNotNone(connection._client)
         self.assertIsNotNone(connection._bqstorage_client)
 
-    @unittest.skipIf(
-        bigquery_storage is None, "Requires `google-cloud-bigquery-storage`"
-    )
     def test_connect_w_client(self):
         from google.cloud.bigquery.dbapi import connect
         from google.cloud.bigquery.dbapi import Connection
@@ -108,9 +97,6 @@ class TestConnection(unittest.TestCase):
         self.assertIs(connection._client, mock_client)
         self.assertIs(connection._bqstorage_client, mock_bqstorage_client)
 
-    @unittest.skipIf(
-        bigquery_storage is None, "Requires `google-cloud-bigquery-storage`"
-    )
     def test_connect_w_both_clients(self):
         from google.cloud.bigquery.dbapi import connect
         from google.cloud.bigquery.dbapi import Connection
@@ -144,9 +130,6 @@ class TestConnection(unittest.TestCase):
             ):
                 getattr(connection, method)()
 
-    @unittest.skipIf(
-        bigquery_storage is None, "Requires `google-cloud-bigquery-storage`"
-    )
     def test_close_closes_all_created_bigquery_clients(self):
         client = self._mock_client()
         bqstorage_client = self._mock_bqstorage_client()
@@ -169,9 +152,6 @@ class TestConnection(unittest.TestCase):
         self.assertTrue(client.close.called)
         self.assertTrue(bqstorage_client._transport.grpc_channel.close.called)
 
-    @unittest.skipIf(
-        bigquery_storage is None, "Requires `google-cloud-bigquery-storage`"
-    )
     def test_close_does_not_close_bigquery_clients_passed_to_it(self):
         client = self._mock_client()
         bqstorage_client = self._mock_bqstorage_client()
